/*
 * Trackedit driver for the OSS /dev/dsp output device.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <audiofile.h>

#include "drivers.h"

/* ======================================================================== */

int drv_file_open(Driver *d) {
/*
 * Opens the specified `file' device and configures it for use with this 
 * program.  Returns 1 on success.
 *
 */
AFfilesetup setup;
AFfilehandle f;
  if (d->dev.file.filename==NULL) {
    fprintf(stderr,"drv_file_open(): no filename specified\n");
    return 0;
  }
  setup = afNewFileSetup();
  afInitFileFormat(setup,d->dev.file.file_format);
  afInitChannels(setup,AF_DEFAULT_TRACK,d->n_channels);
  afInitSampleFormat(setup,AF_DEFAULT_TRACK,d->dev.file.sample_format,
    d->sample_size);
  f = afOpenFile(d->dev.file.filename,"w",setup);
  afFreeFileSetup(setup);
  if (f==NULL) {
    return 0;
  }
  d->dev.file.fh = f;
  return(1);
}
/* ======================================================================== */

int drv_file_close(Driver *d) {
/*
 * Closes the DSP device associated with the given device.
 *
 */
  return afCloseFile(d->dev.file.fh)==0;
}
/* ======================================================================== */

int drv_file_write(Driver *d, char *buf, int size) {
/*
 * Writes `size' bytes from the buffer to the given OSS DSP device.
 *
 */
int nframes = size / (d->n_channels * (d->sample_size>>3));
  return afWriteFrames(d->dev.file.fh,AF_DEFAULT_TRACK,buf,nframes) == nframes;
}
/* ======================================================================== */

int drv_file_free(Driver *d) {
/*
 * Frees the resources used by the given OSS DSP driver, but not the driver
 * object itself.
 *
 */
  if (d->dev.file.filename!=NULL)
    free(d->dev.file.filename);
  return 1;
}
/* ======================================================================== */

int drv_file_init(Driver *d) {
/*
 * Initialises a new file driver instance.  The file's format defaults to
 * 16-bit two's complement WAV file format.
 *
 */
  d->dev.file.filename = NULL;
  d->dev.file.fh = NULL;
  d->dev.file.file_format = AF_FILE_WAVE;
  d->dev.file.sample_format = AF_SAMPFMT_TWOSCOMP;
  d->sample_size = 16;
  d->open = drv_file_open;
  d->write = drv_file_write;
  d->close = drv_file_close;
  d->free = drv_file_free;
  return 1;
}
/* ======================================================================== */
